/* Desenha as arestas de um cubo (usando a tecnica "fio-de-arame") e
	faz rotacoes interativas consecutivas.
	Usa rotacao por matriz 2x2 e calcula rotacao usando long e nao float.
*/

#include <graphics.h>
#include <conio.h>                   // getch () e kbhit
#include <math.h>                    // seno e cosseno
#include <stdarg.h>                  // va_list, va_start, va_arg, va_end
#include <dos.h>                     // delay

#define N_VERTICES 8                 // numero de vertices da figura
#define PI 3.14159265359
#define PROPVID 1


long
	Sen [ 360 ],          // afim de tornar o programa mais rapido: sao
	Cos [ 360 ];          // precalculados os valores dos senos e cossenos e
int
	XO [ N_VERTICES ] = {-50, 50, 50, -50, -50, 50, 50, -50},  // a figura eh
	YO [ N_VERTICES ] = {50, 50, 50, 50, -50, -50, -50, -50},  // definida
	ZO [ N_VERTICES ] = {50, 50, -50, -50, -50, -50, 50, 50},  // globalmente.
	XR [ N_VERTICES ],    // XO, YO, ZO tem os valores originais das
	YR [ N_VERTICES ],    // coordenadas dos vetices do desenho. XR, YR, ZR
	ZR [ N_VERTICES ],    // tem os valores que sao obtidos opos uma rotacao.
	X2D [ N_VERTICES ],   // X2D e Y2D tem os valores dos vertices projetados.
	Y2D [ N_VERTICES ];



/* Calcula senos e cossenos de 0 a 359 graus.
	Como eh mais rapido calcular valores inteiros, o resultado do seno e do
	cosseno eh quardado numa matriz de inteiros (longos = 4 bytes) como um
	numero inteiro. Apos o calculo da rotacao eh feita devida a conversao para
	fracionario. Ha uma pequena perda de precisao, pois a parte fracionaria
	que foi obtida no calculo eh perdida (o numero eh truncado).
****************************************************************************/
void Pre_Calcula ()
{
	for ( int Graus = 0; Graus < 360; Graus ++ )
		{
			Sen [ Graus ] = ( long ) ( sin ( Graus * PI / 180 ) * 65536 );
			Cos [ Graus ] = ( long ) ( cos ( Graus * PI / 180 ) * 65536 );
		}
}



/* Calcula as novas coordenadas da figura, no espaco, segundo um
  certo angulo de rotacao em cada eixo por meio de uma matriz de rotacao.
****************************************************************************/
void Rotaciona ( int Alfa, int Beta, int Teta )
{
	int
		XAux, YAux, ZAux;
	for ( int Conta = 0; Conta < N_VERTICES; Conta ++ )
		{
			XR [ Conta ] = XO [ Conta ];
			YR [ Conta ] = YO [ Conta ];
			ZR [ Conta ] = ZO [ Conta ];
			XAux = ( XR [Conta] * Cos [Alfa] >> 16 ) + ( YR [Conta] * Sen [Alfa] >> 16 );
			YAux = ( YR [Conta] * Cos [Alfa] >> 16 ) - ( XR [Conta] * Sen [Alfa] >> 16 );
			YR [Conta] = ( YAux * Cos [Beta] >> 16 ) + ( ZR [Conta] * Sen [Beta] >> 16 );
			ZAux = ( ZR [Conta] * Cos [Beta] >> 16 ) - ( YAux * Sen [Beta] >> 16 );
			XR [Conta] = ( XAux * Cos [Teta] >> 16 ) + ( ZAux * Sen [Teta] >> 16 );
			ZR [Conta] = ( ZAux * Cos [Teta] >> 16 ) - ( XAux * Sen [Teta] >> 16 );
		}
}



/* Faz a projecao dos pontos em 3D, com perspectiva, para a tela.
	E desloca os pontos para o centro da tela.
	A divisao por Z cria a ideia de que a tela tem profundidade. No caso o
	"ponto de fuga", esta no centro da tela.
***************************************************************************/
void Projeta ( int Distancia, int Perspectiva )
{
	for ( int Conta = 0; Conta < N_VERTICES; Conta ++ )
		{
			X2D [ Conta ] = ( int ) (   ( long ) XR [ Conta ] * Perspectiva / ( ZR [ Conta ] + Perspectiva - Distancia ) )             + 320;
			Y2D [ Conta ] = ( int ) ( ( ( long ) YR [ Conta ] * Perspectiva / ( ZR [ Conta ] + Perspectiva - Distancia ) ) / PROPVID ) + 100;
		}
}



/* Desenha a figura na tela.
****************************************************************************/
void Desenha ( int Marcar ... )
{
	if ( Marcar )               // se Marcar = 1 projetar e desenhar com a
		{                        // maior cor.
			va_list Lista_Param;
			va_start ( Lista_Param, Marcar );
			int
				Distancia = va_arg (Lista_Param, int),
				Perspectiva = va_arg (Lista_Param, int);
			setcolor ( getmaxcolor () );
			Projeta ( Distancia, Perspectiva );
			va_end (Lista_Param);
		}
	else
		setcolor ( 0 );          // senao apagar desenhando com preto.
	line ( X2D [0], Y2D [0], X2D [1], Y2D [1] );
	line ( X2D [0], Y2D [0], X2D [3], Y2D [3] );
	line ( X2D [0], Y2D [0], X2D [7], Y2D [7] );
	line ( X2D [0], Y2D [0], X2D [2], Y2D [2] );
	line ( X2D [1], Y2D [1], X2D [2], Y2D [2] );
	line ( X2D [1], Y2D [1], X2D [6], Y2D [6] );
	line ( X2D [1], Y2D [1], X2D [3], Y2D [3] );
	line ( X2D [2], Y2D [2], X2D [3], Y2D [3] );
	line ( X2D [2], Y2D [2], X2D [5], Y2D [5] );
	line ( X2D [3], Y2D [3], X2D [4], Y2D [4] );
	line ( X2D [4], Y2D [4], X2D [5], Y2D [5] );
	line ( X2D [4], Y2D [4], X2D [7], Y2D [7] );
	line ( X2D [5], Y2D [5], X2D [6], Y2D [6] );
	line ( X2D [6], Y2D [6], X2D [7], Y2D [7] );
}



/* Verifica se os angulos de rotacao estao dentro da faixa de 0 a 359
****************************************************************************/
void Testa_Angulos ( int &Alfa, int &Beta, int &Teta )
{
	Alfa = Alfa % 360;       // calcula o modulo
	Beta = Beta % 360;
	Teta = Teta % 360;
	if ( Alfa < 0 ) Alfa = 359;      // reflete o valor se for negativo
	if ( Beta < 0 ) Beta = 359;
	if ( Teta < 0 ) Teta = 359;
}



main ()
{
	char
		Tecla = 0;
	int
		GrDriver = DETECT,
		GrMode,
		Alfa = 30,
		Beta = 0,
		Teta = 60,
		PRot_Alfa = 1,
		PRot_Beta = 1,
		PRot_Teta = 1,
		Conta_Alfa = 0,
		Conta_Beta = 0,
		Conta_Teta = 0;

	initgraph (&GrDriver, &GrMode, "");
	Pre_Calcula ();
	while ( Tecla != 27 )
		{
			while ( !kbhit () )
				{
					delay (7);
					Desenha ( 0 );
					Testa_Angulos ( Alfa, Beta, Teta );
					Rotaciona ( Alfa, Beta, Teta );
					Desenha ( 1,  50, 700 );
					Conta_Alfa += 1;
					Conta_Beta += 1;
					Conta_Teta += 1;
					if ( PRot_Alfa < Conta_Alfa )
						{
							Conta_Alfa = 0;
							Alfa += 1;
						}
					if ( PRot_Beta < Conta_Beta )
						{
							Conta_Beta = 0;
							Beta += 1;
						}
					if ( PRot_Teta < Conta_Teta )
						{
							Conta_Teta = 0;
							Teta += 1;
						}
				}
			Tecla = getch ();
			if ( ( Tecla == '1' ) && ( PRot_Alfa < 1000 ) ) PRot_Alfa += 1;
			if ( ( Tecla == '2' ) && ( PRot_Alfa > 1 ) ) PRot_Alfa -= 1;
			if ( ( Tecla == '3' ) && ( PRot_Beta < 1000 ) ) PRot_Beta += 1;
			if ( ( Tecla == '4' ) && ( PRot_Beta > 1 ) ) PRot_Beta -= 1;
			if ( ( Tecla == '5' ) && ( PRot_Teta < 1000 ) ) PRot_Teta += 1;
			if ( ( Tecla == '6' ) && ( PRot_Teta > 1 ) ) PRot_Teta -= 1;
		}
	closegraph ();
}